@@ -1,13 +1,15 @@
 # Copyright (c) ONNX Project Contributors
 #
 # SPDX-License-Identifier: Apache-2.0
+from __future__ import annotations
 
 import sys
 
 import numpy as np
 
 import onnx
-from onnx import TensorProto, helper
+import onnx._custom_element_types as custom
+from onnx import TensorProto, helper, subbyte
 from onnx.backend.test.case.base import Base
 from onnx.backend.test.case.node import expect
 from onnx.helper import (
@@ -16,7 +18,11 @@ from onnx.helper import (
     make_tensor,
     tensor_dtype_to_field,
 )
-from onnx.numpy_helper import float8e4m3_to_float32, float8e5m2_to_float32
+from onnx.numpy_helper import (
+    float8e4m3_to_float32,
+    float8e5m2_to_float32,
+    unpacked_float4e2m1_to_float32,
+)
 
 
 class Cast(Base):
@@ -50,10 +56,31 @@ class Cast(Base):
             ("FLOAT8E5M2", "FLOAT16"),
             ("FLOAT8E5M2FNUZ", "FLOAT"),
             ("FLOAT8E5M2FNUZ", "FLOAT16"),
+            ("FLOAT", "UINT4"),
+            ("FLOAT16", "UINT4"),
+            ("FLOAT", "INT4"),
+            ("FLOAT16", "INT4"),
+            ("UINT4", "FLOAT"),
+            ("UINT4", "FLOAT16"),
+            ("UINT4", "UINT8"),
+            ("INT4", "FLOAT"),
+            ("INT4", "FLOAT16"),
+            ("INT4", "INT8"),
+            ("FLOAT4E2M1", "FLOAT"),
+            ("FLOAT4E2M1", "FLOAT16"),
+            ("FLOAT", "FLOAT4E2M1"),
+            ("FLOAT16", "FLOAT4E2M1"),
         ]
 
         vect_float32_to_float8e4m3 = np.vectorize(float32_to_float8e4m3)
         vect_float32_to_float8e5m2 = np.vectorize(float32_to_float8e5m2)
+        vect_float32_to_uint4 = np.vectorize(
+            lambda x: subbyte.float32_to_4bit_unpacked(x, signed=False)
+        )
+        vect_float32_to_int4 = np.vectorize(
+            lambda x: subbyte.float32_to_4bit_unpacked(x, signed=True)
+        )
+
         f8_types = ("FLOAT8E4M3FN", "FLOAT8E4M3FNUZ", "FLOAT8E5M2", "FLOAT8E5M2FNUZ")
 
         for from_type, to_type in test_cases:
@@ -124,6 +151,9 @@ class Cast(Base):
                         "INF",
                         "+INF",
                         "-INF",
+                        "-0.0000001",
+                        "0.0000001",
+                        "-1000000",
                     ],
                     dtype=np.float32,
                 )
@@ -131,33 +161,33 @@ class Cast(Base):
                 if from_type == "FLOAT":
                     input_values = np_fp32
                     input = make_tensor(
-                        "x", TensorProto.FLOAT, [3, 4], np_fp32.tolist()
+                        "x", TensorProto.FLOAT, [3, 5], np_fp32.tolist()
                     )
                 elif from_type == "FLOAT16":
                     input_values = np_fp32.astype(np.float16).astype(np.float32)
                     input = make_tensor(
-                        "x", TensorProto.FLOAT16, [3, 4], input_values.tolist()
+                        "x", TensorProto.FLOAT16, [3, 5], input_values.tolist()
                     )
                 elif from_type == "FLOAT8E4M3FN":
                     input_values = float8e4m3_to_float32(
                         vect_float32_to_float8e4m3(np_fp32)
                     )
                     input = make_tensor(
-                        "x", TensorProto.FLOAT8E4M3FN, [3, 4], input_values.tolist()
+                        "x", TensorProto.FLOAT8E4M3FN, [3, 5], input_values.tolist()
                     )
                 elif from_type == "FLOAT8E4M3FNUZ":
                     input_values = float8e4m3_to_float32(
                         vect_float32_to_float8e4m3(np_fp32, uz=True), uz=True
                     )
                     input = make_tensor(
-                        "x", TensorProto.FLOAT8E4M3FNUZ, [3, 4], input_values.tolist()
+                        "x", TensorProto.FLOAT8E4M3FNUZ, [3, 5], input_values.tolist()
                     )
                 elif from_type == "FLOAT8E5M2":
                     input_values = float8e5m2_to_float32(
                         vect_float32_to_float8e5m2(np_fp32)
                     )
                     input = make_tensor(
-                        "x", TensorProto.FLOAT8E5M2, [3, 4], input_values.tolist()
+                        "x", TensorProto.FLOAT8E5M2, [3, 5], input_values.tolist()
                     )
                 elif from_type == "FLOAT8E5M2FNUZ":
                     input_values = float8e5m2_to_float32(
@@ -166,7 +196,7 @@ class Cast(Base):
                         uz=True,
                     )
                     input = make_tensor(
-                        "x", TensorProto.FLOAT8E5M2FNUZ, [3, 4], input_values.tolist()
+                        "x", TensorProto.FLOAT8E5M2FNUZ, [3, 5], input_values.tolist()
                     )
                 else:
                     raise ValueError(
@@ -200,10 +230,113 @@ class Cast(Base):
                         "Conversion from {from_type} to {to_type} is not tested."
                     )
                 expected_tensor = make_tensor(
-                    "x", getattr(TensorProto, to_type), [3, 4], expected.tolist()
+                    "x", getattr(TensorProto, to_type), [3, 5], expected.tolist()
                 )
                 output = expected_tensor
+            elif from_type in ("UINT4", "INT4") or to_type in ("UINT4", "INT4"):
+                np_fp32 = np.arange(-9, 16).astype(np.float32)
+                input_shape = (5, 5)
+                if from_type == "FLOAT":
+                    input_values = np_fp32
+                    input = make_tensor(
+                        "x", TensorProto.FLOAT, input_shape, input_values.tolist()
+                    )
+                elif from_type == "FLOAT16":
+                    input_values = np_fp32.astype(np.float16)
+                    input = make_tensor(
+                        "x", TensorProto.FLOAT16, input_shape, input_values.tolist()
+                    )
+                elif from_type == "UINT4":
+                    input_values = vect_float32_to_uint4(np_fp32)
+                    input = make_tensor(
+                        "x", TensorProto.UINT4, input_shape, input_values.tolist()
+                    )
+                elif from_type == "INT4":
+                    input_values = vect_float32_to_int4(np_fp32)
+                    input = make_tensor(
+                        "x", TensorProto.INT4, input_shape, input_values.tolist()
+                    )
+                else:
+                    raise ValueError(
+                        "Conversion from {from_type} to {to_type} is not tested."
+                    )
+                if to_type == "UINT4":
+                    expected = vect_float32_to_uint4(input_values).astype(custom.uint4)
+                elif to_type == "INT4":
+                    expected = vect_float32_to_int4(input_values).astype(custom.int4)
+                elif to_type == "FLOAT16":
+                    expected = input_values.astype(np.float16)
+                elif to_type == "FLOAT":
+                    expected = input_values
+                elif to_type == "UINT8":
+                    expected = input_values.astype(np.uint8)
+                elif to_type == "INT8":
+                    expected = input_values.astype(np.int8)
+                else:
+                    raise ValueError(
+                        "Conversion from {from_type} to {to_type} is not tested."
+                    )
+                expected_tensor = make_tensor(
+                    "y", getattr(TensorProto, to_type), input_shape, expected.tolist()
+                )
+                output = expected_tensor
+                input_type_proto = onnx.helper.make_tensor_type_proto(
+                    getattr(TensorProto, from_type), input_shape
+                )
+                output_type_proto = onnx.helper.make_tensor_type_proto(
+                    getattr(TensorProto, to_type), input_shape
+                )
+            elif from_type == "FLOAT4E2M1" or to_type == "FLOAT4E2M1":
+                np_fp32 = np.array(
+                    [
+                        "0.48",
+                        "0.25",
+                        "1.05",
+                        "-3.5",
+                        "-8",
+                        "9",
+                        "1000000",
+                        "1e-7",
+                        "NaN",
+                        "INF",
+                        "+INF",
+                        "-INF",
+                        "-4",
+                        "0.01",
+                        "-0.0",
+                    ],
+                    dtype=np.float32,
+                )
+                input_shape = (3, 5)
+                if from_type == "FLOAT":
+                    input_values = np_fp32
+                    input = make_tensor(
+                        "x", TensorProto.FLOAT, input_shape, input_values.tolist()
+                    )
+                elif from_type == "FLOAT16":
+                    input_values = np_fp32.astype(np.float16).astype(np.float32)
+                    input = make_tensor(
+                        "x", TensorProto.FLOAT16, input_shape, input_values.tolist()
+                    )
+                elif from_type == "FLOAT4E2M1":
+                    input = make_tensor(
+                        "x", TensorProto.FLOAT4E2M1, input_shape, np_fp32.tolist()
+                    )
+                else:
+                    raise ValueError(
+                        f"Conversion from {from_type} to {to_type} is not tested."
+                    )
 
+                if to_type not in ("FLOAT", "FLOAT16", "FLOAT4E2M1"):
+                    raise ValueError(
+                        f"Conversion from {from_type} to {to_type} is not tested."
+                    )
+                expected = unpacked_float4e2m1_to_float32(
+                    subbyte.float32_to_float4e2m1_unpacked(np_fp32)
+                )
+                output = make_tensor(
+                    "y", getattr(TensorProto, to_type), input_shape, expected.tolist()
+                )
             elif from_type != "STRING":
                 input = np.random.random_sample(shape).astype(
                     helper.tensor_dtype_to_np_dtype(getattr(TensorProto, from_type))
@@ -295,17 +428,20 @@ class Cast(Base):
                     "INF",
                     "+INF",
                     "-INF",
+                    "-0.0000001",
+                    "0.0000001",
+                    "-1000000",
                 ],
                 dtype=np.float32,
             )
 
             if from_type == "FLOAT":
                 input_values = np_fp32
-                input = make_tensor("x", TensorProto.FLOAT, [3, 4], np_fp32.tolist())
+                input = make_tensor("x", TensorProto.FLOAT, [3, 5], np_fp32.tolist())
             elif from_type == "FLOAT16":
                 input_values = np_fp32.astype(np.float16).astype(np.float32)
                 input = make_tensor(
-                    "x", TensorProto.FLOAT16, [3, 4], input_values.tolist()
+                    "x", TensorProto.FLOAT16, [3, 5], input_values.tolist()
                 )
             else:
                 raise ValueError(
@@ -333,7 +469,7 @@ class Cast(Base):
             tensor = TensorProto()
             tensor.data_type = getattr(TensorProto, to_type)
             tensor.name = "x"
-            tensor.dims.extend([3, 4])
+            tensor.dims.extend([3, 5])
             field = tensor_dtype_to_field(tensor.data_type)
             getattr(tensor, field).extend(ivals)
 
