@@ -2,12 +2,12 @@
  * SPDX-License-Identifier: Apache-2.0
  */
 
-#include "onnx/defs/function.h"
-#include "onnx/defs/schema.h"
-
 #include <algorithm>
 #include <numeric>
 
+#include "onnx/defs/function.h"
+#include "onnx/defs/schema.h"
+
 namespace ONNX_NAMESPACE {
 
 static const char* SequenceEmpty_ver11_doc = R"DOC(
@@ -81,7 +81,7 @@ ONNX_OPERATOR_SET_SCHEMA(
 
           output_tensor_type->set_elem_type(static_cast<TensorProto_DataType>(input_elem_types[0]));
 
-          if (!hasNInputShapes(ctx, static_cast<int>(numInputs))) {
+          if (!hasNInputShapes(ctx, numInputs)) {
             return;
           }
 
@@ -188,7 +188,7 @@ ONNX_OPERATOR_SET_SCHEMA(
         .TypeAndShapeInferenceFunction([](InferenceContext& ctx) {
           const auto input0_type = ctx.getInputType(0);
           if (nullptr == input0_type) {
-            fail_type_inference("Input type for input at index 0 is null. Type info is expected.")
+            fail_type_inference("Input type for input at index 0 is null. Type info is expected.");
           }
           ctx.getOutputType(0)->CopyFrom(input0_type->sequence_type().elem_type());
         }));
@@ -345,17 +345,17 @@ ONNX_OPERATOR_SET_SCHEMA(
 
               std::vector<int64_t> splitSizes;
               if (splitInitializer->data_type() == TensorProto::INT64) {
-                const auto& data = ParseData<int64_t>(splitInitializer);
+                const auto data = ParseData<int64_t>(splitInitializer);
                 splitSizes.insert(splitSizes.end(), data.begin(), data.end());
               } else if (splitInitializer->data_type() == TensorProto::INT32) {
-                const auto& data = ParseData<int32_t>(splitInitializer);
+                const auto data = ParseData<int32_t>(splitInitializer);
                 splitSizes.insert(splitSizes.end(), data.begin(), data.end());
               } else {
                 // unaccepted data type
                 fail_shape_inference("Only supports `int32_t` or `int64_t` inputs for split");
               }
 
-              if (splitSizes.size() == 0) {
+              if (splitSizes.empty()) {
                 fail_shape_inference("Input 'split' can not be empty.");
               }
 
@@ -526,7 +526,7 @@ the input.
 This operator assumes that processing each sample is independent and could executed in parallel
 or in any order. Users cannot expect any specific ordering in which each subgraph is computed.)DOC";
 
-void SequenceMapInferenceFunction(InferenceContext& ctx) {
+static void SequenceMapInferenceFunction(InferenceContext& ctx) {
   auto num_inputs = ctx.getNumInputs();
   assert(num_inputs > 0);
 
@@ -576,10 +576,8 @@ void SequenceMapInferenceFunction(InferenceContext& ctx) {
   }
 }
 
-bool BuildSequenceMapBodyFunc(
-    const FunctionBodyBuildContext& ctx,
-    const OpSchema& schema,
-    FunctionProto& functionProto) {
+static bool
+BuildSequenceMapBodyFunc(const FunctionBodyBuildContext& ctx, const OpSchema& schema, FunctionProto& functionProto) {
   schema.BuildFunction(functionProto);
 
   // variadic input/outputs will be expanded
@@ -609,9 +607,9 @@ bool BuildSequenceMapBodyFunc(
   if (!first_input_type->has_sequence_type())
     ONNX_THROW_EX(std::invalid_argument("Expected a sequence type for input 0"));
 
-  auto schema_inputs = schema.inputs();
-  auto input_0_name = schema_inputs[0].GetName();
-  auto input_1_name = schema_inputs[1].GetName(); // variadic input
+  auto const& schema_inputs = schema.inputs();
+  auto const& input_0_name = schema_inputs[0].GetName();
+  auto const& input_1_name = schema_inputs[1].GetName(); // variadic input
 
   *functionProto.add_input() = input_0_name;
   for (int i = 1; i < ninputs; i++) {
@@ -620,7 +618,7 @@ bool BuildSequenceMapBodyFunc(
     *functionProto.add_input() = MakeString(input_1_name, "_", i);
   }
 
-  auto schema_outputs = schema.outputs();
+  auto const& schema_outputs = schema.outputs();
   auto output_0_name = schema_outputs[0].GetName();
   for (int i = 0; i < noutputs; i++) {
     if (!ctx.hasOutput(i))
@@ -676,7 +674,7 @@ bool BuildSequenceMapBodyFunc(
         seq_at_node.add_input(functionProto.input(inputIndex));
         seq_at_node.add_input(iter_count_name);
         seq_at_node.add_output(g_inputs.Get(inputIndex).name());
-        *loopbody_graph.add_node() = seq_at_node;
+        *loopbody_graph.add_node() = std::move(seq_at_node);
       } else {
         // If not a sequence, simply connect
         NodeProto identity;
@@ -684,7 +682,7 @@ bool BuildSequenceMapBodyFunc(
         identity.set_op_type("Identity");
         identity.add_input(functionProto.input(inputIndex));
         identity.add_output(g_inputs.Get(inputIndex).name());
-        *loopbody_graph.add_node() = identity;
+        *loopbody_graph.add_node() = std::move(identity);
       }
     }
 
@@ -743,8 +741,8 @@ bool BuildSequenceMapBodyFunc(
     std::string seqempty_name = MakeString(out_prefix, "_seqempty");
     int64_t dtype = g_outputs.Get(outputIndex).type().tensor_type().elem_type();
     nodes.push_back({{seqempty_name}, "SequenceEmpty", {}, {MakeAttribute("dtype", dtype)}});
-    loop_node_inputs.push_back(seqempty_name);
-    loop_node_outputs.push_back(output_name);
+    loop_node_inputs.emplace_back(std::move(seqempty_name));
+    loop_node_outputs.emplace_back(std::move(output_name));
   }
 
   nodes.push_back({loop_node_outputs, "Loop", loop_node_inputs, {MakeAttribute("body", loopbody_graph)}});
