@@ -1,7 +1,7 @@
 # Copyright (c) ONNX Project Contributors
 
 # SPDX-License-Identifier: Apache-2.0
-# pylint: disable=W0221
+from __future__ import annotations
 
 import numpy as np
 
@@ -12,7 +12,7 @@ class PRelu(OpRun):
     def _run(self, x, slope):  # type: ignore
         try:
             return (np.where(x > 0, x, x * slope).astype(x.dtype),)
-        except ValueError as e:
+        except ValueError:
             # Broadcast did not work according to numpy.
             # The logic is then the following, if slope has d elements,
             # the following code is looking for d in x.shape. If it is found
@@ -31,4 +31,4 @@ class PRelu(OpRun):
                 if n == 1:
                     xs = x * slope.reshape(tuple(new_shape))
                     return (np.where(x > 0, x, xs).astype(x.dtype),)
-            raise e
+            raise
