@@ -1,7 +1,7 @@
 # Copyright (c) ONNX Project Contributors
 
 # SPDX-License-Identifier: Apache-2.0
-# pylint: disable=W0221
+from __future__ import annotations
 
 import numpy as np
 
@@ -12,15 +12,15 @@ def _scatter_nd_impl(data, indices, updates, reduction=None):  # type: ignore
     output = np.copy(data)
     for i in np.ndindex(indices.shape[:-1]):
         if reduction == "add":
-            output[indices[i]] += updates[i]
+            output[tuple(indices[i])] += updates[i]
         elif reduction == "mul":
-            output[indices[i]] *= updates[i]
+            output[tuple(indices[i])] *= updates[i]
         elif reduction == "max":
-            output[indices[i]] = np.maximum(output[indices[i]], updates[i])
+            output[tuple(indices[i])] = np.maximum(output[indices[i]], updates[i])
         elif reduction == "min":
-            output[indices[i]] = np.minimum(output[indices[i]], updates[i])
+            output[tuple(indices[i])] = np.minimum(output[indices[i]], updates[i])
         else:
-            output[indices[i]] = updates[i]
+            output[tuple(indices[i])] = updates[i]
     return output
 
 
