@@ -4,7 +4,7 @@
 from __future__ import annotations
 
 import unittest
-from typing import Callable, Sequence
+from typing import TYPE_CHECKING, Callable
 
 import numpy as np
 
@@ -23,6 +23,9 @@ from onnx import (
     version_converter,
 )
 
+if TYPE_CHECKING:
+    from collections.abc import Sequence
+
 
 def _load_model(m_def: str) -> ModelProto:
     """Parses a model from a string representation, including checking the model for correctness"""
