@@ -1,16 +1,19 @@
+// Copyright (c) ONNX Project Contributors
+
 /*
  * SPDX-License-Identifier: Apache-2.0
  */
 
 #include <iostream>
 #include <set>
+
 #include "gtest/gtest.h"
 #include "onnx/checker.h"
 #include "onnx/common/constants.h"
+#include "onnx/defs/function.h"
 #include "onnx/defs/parser.h"
+#include "onnx/defs/printer.h"
 #include "onnx/defs/schema.h"
-#include "onnx/onnx-operators_pb.h"
-#include "onnx/onnx_pb.h"
 #include "onnx/shape_inference/implementation.h"
 
 namespace ONNX_NAMESPACE {
@@ -18,7 +21,7 @@ namespace Test {
 using namespace checker;
 using TENSOR_TYPES_MAP = std::unordered_map<std::string, std::vector<std::string>>;
 
-void GetFunctionProtoOpsetImport(
+static void GetFunctionProtoOpsetImport(
     const OpSchema& op,
     const FunctionProto* function_proto,
     std::unordered_map<std::string, int>& op_set) {
@@ -31,13 +34,13 @@ void GetFunctionProtoOpsetImport(
   }
 }
 
-void VerifyTypeConstraint(const OpSchema& function_op, const FunctionProto* function_proto, int& counter) {
+static void VerifyTypeConstraint(const OpSchema& function_op, const FunctionProto* function_proto, int& counter) {
   // This is a simple partial type-checker for a function-body.
   // TODO: Revisit to make the type-checker more complete.
   TENSOR_TYPES_MAP tc_map;
   std::set<std::string> primitive_types(OpSchema::all_tensor_types().begin(), OpSchema::all_tensor_types().end());
   for (const auto& input : function_op.inputs()) {
-    std::string name = input.GetName();
+    const std::string& name = input.GetName();
     auto& tvec = tc_map[name];
     for (const auto& t : input.GetTypes()) {
       tvec.emplace_back(*t);
@@ -45,7 +48,7 @@ void VerifyTypeConstraint(const OpSchema& function_op, const FunctionProto* func
   }
 
   for (const auto& output : function_op.outputs()) {
-    std::string name = output.GetName();
+    const std::string& name = output.GetName();
     auto& tvec = tc_map[name];
     for (const auto& t : output.GetTypes()) {
       tvec.emplace_back(*t);
@@ -56,7 +59,7 @@ void VerifyTypeConstraint(const OpSchema& function_op, const FunctionProto* func
   GetFunctionProtoOpsetImport(function_op, function_proto, op_set);
 
   for (auto& node : function_proto->node()) {
-    std::string op_type = node.op_type();
+    const std::string& op_type = node.op_type();
     std::unordered_map<std::string, int>::const_iterator it = op_set.find(node.domain());
     if (it == op_set.end()) {
       fail_check(
@@ -73,7 +76,7 @@ void VerifyTypeConstraint(const OpSchema& function_op, const FunctionProto* func
     auto num_actual_inputs = static_cast<size_t>(node.input_size());
 
     for (size_t i = 0; i < num_actual_inputs; ++i) {
-      auto actual_param_name = node.input(static_cast<int>(i));
+      const auto& actual_param_name = node.input(static_cast<int>(i));
       auto iter = tc_map.find(actual_param_name);
       if (iter != tc_map.end()) {
         // if i >= num_formal_inputs, it is a variadic parameter corresponding
@@ -87,8 +90,13 @@ void VerifyTypeConstraint(const OpSchema& function_op, const FunctionProto* func
         for (auto& actual_type : iter->second) {
           if (allowed_types.find(actual_type) == allowed_types.end()) {
             fail_check(
-                "Input type " + actual_type + " of parameter " + actual_param_name + " of function " +
-                function_op.Name() + " is not allowed by operator " + op_type);
+                "Input type ",
+                actual_type,
+                " of parameter ",
+                actual_param_name + " of function ",
+                function_op.Name(),
+                " is not allowed by operator ",
+                op_type);
           }
         }
       }
@@ -102,7 +110,177 @@ void VerifyTypeConstraint(const OpSchema& function_op, const FunctionProto* func
   ++counter;
 }
 
-void VerifyFunction(const OpSchema& op, const FunctionProto* function_proto, int& counter) {
+// Testing the function-definitions provided for function-ops in ONNX schema registry.
+// We type-check the function-definition for all possible input-typings, as permitted
+// by the op-schema. Since the type-checking is dependent on attribute-values, we specify
+// the attribute-values for which we want to do the testing down below.
+
+// The set of attribute-values (for testing a function) is represented using a vector.
+using AttributeValues = std::vector<AttributeProto>;
+
+// FunctionOpAttributeMap: Used to implement a map from OpSchema to a set of AttributeValues
+// (implemented as a vector). The testing will be done for each attribute-values specified.
+
+struct FunctionOpAttributeMap {
+  std::unordered_map<std::string, std::vector<AttributeValues>> map;
+
+  std::string key(const std::string& domain, const std::string& opname, int opset_version) const {
+    return domain + ":" + opname + ":" + std::to_string(opset_version);
+  }
+
+  void addTestCase(const std::string& opname, int opset_version, std::initializer_list<const char*> attributes) {
+    auto& schema_test_cases = map[key("", opname, opset_version)];
+    schema_test_cases.emplace_back();
+    auto& test_case = schema_test_cases.back();
+    for (auto attr_text : attributes) {
+      test_case.emplace_back();
+      OnnxParser::Parse(test_case.back(), attr_text);
+    }
+  }
+
+  FunctionOpAttributeMap() {
+    addTestCase("Elu", 6, {"alpha = 1.0"});
+    addTestCase("LeakyRelu", 16, {"alpha = 0.1"});
+    addTestCase("HardSigmoid", 6, {"alpha = 0.2", "beta=0.5"});
+    addTestCase("Selu", 6, {"alpha = 1.6", "gamma=1.05"});
+    addTestCase("ReduceL1", 18, {}); // Use default-value for attributes
+    addTestCase("ReduceL1", 18, {"keepdims = 0"});
+    addTestCase("ReduceL1", 18, {"noop_with_empty_axes = 1"});
+    addTestCase("ReduceL2", 18, {});
+    addTestCase("ReduceL2", 18, {"noop_with_empty_axes = 1", "keepdims = 0"});
+    addTestCase("ReduceSumSquare", 18, {});
+    addTestCase("ReduceLogSumExp", 18, {});
+    addTestCase("ThresholdedRelu", 10, {"alpha = 0.9"});
+    addTestCase("HannWindow", 17, {"output_datatype = 1", "periodic = 1"});
+    addTestCase("HammingWindow", 17, {"output_datatype = 1", "periodic = 1"});
+    addTestCase("BlackmanWindow", 17, {"output_datatype = 1", "periodic = 1"});
+    addTestCase("MeanValueNormalization", 13, {});
+    addTestCase("AffineGrid", 20, {"align_corners = 0"});
+    addTestCase("AffineGrid", 20, {"align_corners = 1"});
+
+    // The following test-cases fails, correctly so: Some clarification/changes required
+    // to handle unsigned integers or similar issues:
+    // addTestCase("Shrink", 9, {"bias = 0.0", "lambd = 0.5"});
+    // addTestCase("ReduceLogSum", 18, {});
+    // addTestCase("Range", 11, {});
+
+    // The following test-case fails because the checker doesn't support handling of
+    // default-values of attributes of function-ops
+    // addTestCase("ThresholdedRelu", 10, {});
+  }
+
+  const std::vector<AttributeValues>& getTestCases(const OpSchema& schema) {
+    auto key_value = key(schema.domain(), schema.Name(), schema.SinceVersion());
+    auto it = map.find(key_value);
+    if (it != map.end())
+      return it->second;
+    if (schema.attributes().empty()) {
+      // Test with no-attributes
+      map[key_value].emplace_back();
+    }
+    return map[key_value];
+  }
+
+  static FunctionOpAttributeMap& instance() {
+    static FunctionOpAttributeMap _instance;
+    return _instance;
+  }
+};
+
+struct FunctionTypeChecker {
+  const OpSchema& schema;
+  const FunctionProto& function_proto;
+  const std::vector<AttributeValues>* attribute_cases;
+
+  FunctionTypeChecker(const OpSchema& op_schema, const FunctionProto& proto)
+      : schema(op_schema),
+        function_proto(proto),
+        attribute_cases(&FunctionOpAttributeMap::instance().getTestCases(op_schema)) {}
+
+  // Binds each type-variable in schema to a type-value
+  std::unordered_map<std::string, DataType> typeVarBindings;
+
+  std::vector<std::string> errors;
+
+  void recordError(const std::string& error, const AttributeValues& attrs) {
+    std::ostringstream ostr;
+    ostr << "Type checking failed for instantiation " << schema.Name() << ":" << schema.SinceVersion() << " {";
+    for (auto& pair : typeVarBindings) {
+      ostr << pair.first << " = " << *pair.second << ", ";
+    }
+    for (auto& attr : attrs) {
+      ostr << attr << ", ";
+    }
+    ostr << "}\n" << error << "\n";
+    errors.push_back(ostr.str());
+  }
+
+  void recordSuccess(const AttributeValues& attrs) {
+    std::cout << "Type checking succeeded for instantiation " << schema.Name() << ":" << schema.SinceVersion() << " {";
+    for (auto& pair : typeVarBindings) {
+      std::cout << pair.first << " = " << *pair.second << ", ";
+    }
+    for (auto& attr : attrs) {
+      std::cout << attr << ", ";
+    }
+    std::cout << "}\n";
+  }
+
+  // forTypeVar: This is used to iterate through all possible bindings of type-values
+  // to all type-variables used in the op schema, and invoke the type-checker for
+  // each possible instantiation.
+  void forTypeVar(int i) {
+    auto& typeConstraintVector = schema.typeConstraintParams();
+    if (i < typeConstraintVector.size()) {
+      std::string typeVar = typeConstraintVector[i].type_param_str;
+      auto& values = schema.typeConstraintMap().at(typeVar).first;
+      for (auto typeValue : values) {
+        typeVarBindings[typeVar] = typeValue;
+        // Now, process remaining type-variables
+        forTypeVar(i + 1);
+      }
+    } else {
+      // Generated a complete instantiation of type-values to all type-variables.
+      // Now, check for this instantiation.
+      typeCheckBinding();
+    }
+  }
+
+  // typeCheckBinding: Type-check the function-body for the current type-instantiation
+  void typeCheckBinding() {
+    std::vector<TypeProto> input_types;
+    for (const auto& input : schema.inputs()) {
+      DataType datatype = (1 == input.GetTypes().size())
+          ?
+          // Select the single possible type
+          (*(input.GetTypes().begin()))
+          :
+          // Select the type bound to the type-var in current instantiation
+          typeVarBindings[input.GetTypeStr()];
+      input_types.push_back(Utils::DataTypeUtils::ToTypeProto(datatype));
+    }
+
+    for (auto& attribute_vals : *attribute_cases) {
+      ONNX_TRY {
+        shape_inference::InferFunctionOutputTypes(function_proto, input_types, attribute_vals);
+      }
+      ONNX_CATCH(ONNX_NAMESPACE::InferenceError & e) {
+        ONNX_HANDLE_EXCEPTION(([&]() { recordError(e.what(), attribute_vals); }));
+      }
+    }
+  }
+
+  std::string checkAll() {
+    if (!attribute_cases->empty())
+      forTypeVar(0);
+    std::string all_errors = "";
+    for (const std::string& error : errors)
+      all_errors += error;
+    return all_errors;
+  }
+};
+
+static void VerifyFunction(const OpSchema& op, const FunctionProto* function_proto, int& counter) {
   // Verify function proto is valid
   if (!function_proto) {
     fail_check("Cannot get function body for op '", op.Name(), "'");
@@ -128,6 +306,11 @@ void VerifyFunction(const OpSchema& op, const FunctionProto* function_proto, int
   // Verify function op has compatible Type constraints defined in
   // op and function body.
   VerifyTypeConstraint(op, function_proto, counter);
+
+  FunctionTypeChecker type_checker(op, *function_proto);
+  auto type_errors = type_checker.checkAll();
+  auto success = (type_errors.empty());
+  ASSERT_TRUE(success) << type_errors;
 }
 
 // Verify registered ops with function body has compatible
@@ -135,7 +318,7 @@ void VerifyFunction(const OpSchema& op, const FunctionProto* function_proto, int
 TEST(FunctionVerification, VerifyFunctionOps) {
   const std::vector<OpSchema> schemas = OpSchemaRegistry::get_all_schemas();
   int function_counter = 0, verified_counter = 0;
-  for (const auto s : schemas) {
+  for (const auto& s : schemas) {
     if (!s.HasFunction())
       continue;
     // Skip test for functions with known errors that need to be fixed:
@@ -151,11 +334,11 @@ TEST(FunctionVerification, VerifyFunctionOps) {
         VerifyFunction(s, function_body, verified_counter);
       }
     }
-    ONNX_CATCH(ONNX_NAMESPACE::checker::ValidationError e) {
+    ONNX_CATCH(const ONNX_NAMESPACE::checker::ValidationError& e) {
       ONNX_HANDLE_EXCEPTION([&]() { FAIL() << e.what(); });
     }
   }
-  std::cerr << "[          ] Verified " << verified_counter << "/" << function_counter << " Functions." << std::endl;
+  std::cerr << "[          ] Verified " << verified_counter << "/" << function_counter << " Functions." << '\n';
 }
 
 // Verify that FunctionExpandHelper obtains missing default attributes
@@ -173,7 +356,7 @@ TEST(FunctionVerification, VerifyFunctionExpandHelper) {
 
   for (const auto& node : graph.node()) {
     if (node.op_type() == "ReduceMean") {
-      auto attr = node.attribute(0);
+      const auto& attr = node.attribute(0);
       EXPECT_EQ(attr.name(), "axes");
       EXPECT_EQ(attr.ints().size(), default_axes_attribute.ints().size());
 
@@ -187,7 +370,7 @@ TEST(FunctionVerification, VerifyFunctionExpandHelper) {
          << "the default attribute `axes` has not been assigned to ReduceMean op.";
 }
 
-void RegisterFunctionSchema() {
+static void RegisterFunctionSchema() {
   ONNX_NAMESPACE::OpSchema function_schema;
   function_schema.SetName("DynamicQuantizeLinear_Fake")
       .SetDomain(AI_ONNX_ML_DOMAIN)
