@@ -7,9 +7,7 @@ import itertools
 import os
 import platform
 import unittest
-from typing import Any, Sequence
-
-import numpy
+from typing import TYPE_CHECKING, Any
 
 import onnx.backend.base
 import onnx.backend.test
@@ -19,6 +17,11 @@ from onnx import ModelProto, NodeProto, TensorProto
 from onnx.backend.base import Device, DeviceType
 from onnx.backend.test.runner import BackendIsNotSupposedToImplementIt
 
+if TYPE_CHECKING:
+    from collections.abc import Sequence
+
+    import numpy
+
 # The following just executes the fake backend through the backend test
 # infrastructure. Since we don't have full reference implementation of all ops
 # in ONNX repo, it's impossible to produce the proper results. However, we can
@@ -81,9 +84,7 @@ class DummyBackend(onnx.backend.base.Backend):
     @classmethod
     def supports_device(cls, device: str) -> bool:
         d = Device(device)
-        if d.type == DeviceType.CPU:
-            return True
-        return False
+        return d.type == DeviceType.CPU  # type: ignore[no-any-return]
 
 
 test_coverage_safelist = {
